package com.jz.agent.client.service;




import com.google.gson.Gson;
import com.jz.agent.client.DmpDsAgentClient;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DmpSqlServerAgentClientService implements DmpDsAgentClient {

    private Gson gson = new Gson();

    @Override
    public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {

        // 连接数据库
        String jdbcUrl = ds.getJdbcUrl();
        String userName = ds.getUserName();
        String passwd = ds.getPassword();
        Connection conn = null;
        try {
            conn = DriverManager.getConnection(jdbcUrl, userName, passwd);
            return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
        } catch (SQLException e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "SQLSERVER连通性异常：" + e.getMessage());
        } finally {
            if (null != conn) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    @Override
    public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
        // 连接数据库
        String jdbcUrl = ds.getJdbcUrl();
        String userName = ds.getUserName();
        String passwd = ds.getPassword();
        Connection conn = null;
        List<String> tableNameList = new ArrayList<>();

        try {
            conn = DriverManager.getConnection(jdbcUrl, userName, passwd);
            final DatabaseMetaData metaData = conn.getMetaData();
            final ResultSet res = metaData.getTables(conn.getCatalog(), null, null, new String[]{"TABLE"});
            while (res.next()) {
                String schema = res.getString("TABLE_SCHEM");
                tableNameList.add(schema+"."+res.getString("TABLE_NAME").toLowerCase());
//                if ("dbo".equals(schema)) {
//                    tableNameList.add(res.getString("TABLE_NAME").toLowerCase());
//                }
            }
        } catch (SQLException e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "SQLSERVER连通性异常：" + e.getMessage());
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(tableNameList));
    }

    @Override
    public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {

        List<Map<String, Object>> columnMapList = new ArrayList<>();
        // 连接数据库
        String jdbcUrl = ds.getJdbcUrl();
        String userName = ds.getUserName();
        String passwd = ds.getPassword();
        Connection conn = null;
        System.out.println("tableName=======:"+tableName);
        String[] stableName = tableName.split("\\.");
        System.out.println(stableName[0]);
	    String s = stableName[stableName.length-1];
        try {
            conn = DriverManager.getConnection(jdbcUrl, userName, passwd);
            final DatabaseMetaData metaData = conn.getMetaData();
            ResultSet columns = metaData.getColumns(null, "%", s, "%");

            Map<String, Object> columnMap = null;
            while (columns.next()) {
                columnMap = new HashMap<>();
                String columnName = columns.getString("COLUMN_NAME");
                String typeName = columns.getString("TYPE_NAME");

                columnMap.put("name", columnName.toLowerCase());
                columnMap.put("type", typeName.toUpperCase());
                /*columnMap.put("size", size);
                columnMap.put("scale", dicimalDigits);
                columnMap.put("comment", remarks);*/

                columnMapList.add(columnMap);
            }

        } catch (SQLException e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "SQLSERVER连通性异常：" + e.getMessage());
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(columnMapList));


    }

    @Override
    public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
        Map<String, Object> map = new HashMap<String, Object>();
        // 连接数据库
        String jdbcUrl = ds.getJdbcUrl();
        String userName = ds.getUserName();
        String passwd = ds.getPassword();
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        String[] stableName = tableName.split("\\.");
        tableName = stableName[stableName.length-1];
        try {
            conn = DriverManager.getConnection(jdbcUrl, userName, passwd);
            final DatabaseMetaData metaData = conn.getMetaData();
            ResultSet columns = metaData.getColumns(null, "%", tableName, "%");
            List<String> headerList = new ArrayList<String>();
            while (columns.next()) {
                String columnName = columns.getString("COLUMN_NAME");
                headerList.add(columnName);
            }

            String queryDataSql = "select top 5 * from " + tableName;
            // 建立Statement对象
            stmt = conn.createStatement();
            rs = stmt.executeQuery(queryDataSql);
            List<List<Object>> resultList = new ArrayList<List<Object>>();
            while (rs.next()) {
                List<Object> row = new ArrayList<Object>();
                for (int i = 1; i <= headerList.size(); i++) {
                    row.add(rs.getObject(i));
                }
                resultList.add(row);
            }
            map.put("result", resultList);

        } catch (SQLException e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "SQLSERVER连通性异常：" + e.getMessage());
        } finally {

            if (rs != null){
                try {
                    rs.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }

            if (stmt != null){
                try {
                    stmt.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }

                if (conn != null) {
                    try {
                        conn.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));

    }

}
